/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_BASE_CONNECTION_HPP_
#define _OROPO_BASE_CONNECTION_HPP_

#include <rpc/rpc.h>

namespace oropo
{

struct connection_tcp
{
	static const int socket_type = SOCK_STREAM;
	static const int protocol = IPPROTO_TCP;
	static inline CLIENT *create_client(struct sockaddr_in *addr, unsigned long prognum, unsigned long versnum, int *sockp);
	static inline SVCXPRT *create_service(int sockd);
	static inline bool_t register_service(SVCXPRT *xprt, unsigned long prognum, unsigned long versnum, void (*dispatch)(svc_req *, SVCXPRT *));
};

CLIENT *connection_tcp::create_client(struct sockaddr_in *addr, unsigned long prognum, unsigned long versnum, int *sockp)
{
	return clnttcp_create(addr, prognum, versnum, sockp, 0, 0);
}

SVCXPRT *connection_tcp::create_service(int sockd)
{
	return svctcp_create(sockd, 0, 0);
}

bool_t connection_tcp::register_service(SVCXPRT *xprt, unsigned long prognum, unsigned long versnum, void (*dispatch)(svc_req *, SVCXPRT *))
{
	return svc_register(xprt, prognum, versnum, dispatch, protocol);
}

struct connection_udp
{
	static const int socket_type = SOCK_DGRAM;
	static const int protocol = IPPROTO_UDP;
	static inline CLIENT *create_client(struct sockaddr_in *addr, unsigned long prognum, unsigned long versnum, int *sockp);
	static inline SVCXPRT *create_service(int sockd);
	static inline bool_t register_service(SVCXPRT *xprt, unsigned long prognum, unsigned long versnum, void (*dispatch)(svc_req *, SVCXPRT *));
};

CLIENT *connection_udp::create_client(struct sockaddr_in *addr, unsigned long prognum, unsigned long versnum, int *sockp)
{
	struct timeval timeout;
	timeout.tv_sec = 5;
	timeout.tv_usec = 0;
	return clntudp_create(addr, prognum, versnum, timeout, sockp);
}

SVCXPRT *connection_udp::create_service(int sockd)
{
	return svcudp_create(sockd);
}

bool_t connection_udp::register_service(SVCXPRT *xprt, unsigned long prognum, unsigned long versnum, void (*dispatch)(svc_req *, SVCXPRT *))
{
	return svc_register(xprt, prognum, versnum, dispatch, protocol);
}

template <int PROT>
struct protocol_to_connection
{
};

template <>
struct protocol_to_connection<IPPROTO_TCP>
{
	typedef connection_tcp connection_t;
};

template <>
struct protocol_to_connection<IPPROTO_UDP>
{
	typedef connection_udp connection_t;
};

}

#endif /* #ifndef _OROPO_BASE_CONNECTION_HPP_ */
