/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_BASE_CONSTRUCTOR_HELPER_HPP_
#define _OROPO_BASE_CONSTRUCTOR_HELPER_HPP_

#include <boost/interprocess/managed_shared_memory.hpp>

namespace oropo
{

namespace bi=boost::interprocess;

template <typename ST, typename CT, typename R=ST>
struct constructor_helper
{
	ST m_t;
	template <typename ...Args>
	constructor_helper(const char *p_name, const Args&... p_args) : m_t(CT(), p_name, p_args...) {}
	~constructor_helper() {};
};

template <typename ST, typename R>
struct constructor_helper<ST, bi::create_only_t, R>
{
	ST m_t;
	std::string m_name;
	template <typename ...Args>
	constructor_helper(const char *p_name, const Args&... p_args) : m_t(bi::create_only_t(), p_name, p_args...), m_name(p_name) {}
	~constructor_helper()
	{
		R::remove(m_name.c_str());
	};
};

template <typename ST, typename CT>
struct constructor_helper<ST *, CT, ST *>
{
	ST *m_t;
	template <typename ...Args>
	constructor_helper(bi::managed_shared_memory &p_memory, const Args&... p_args)
	{
		m_t = p_memory.find<ST>(bi::unique_instance).first;
	}
};

template <typename ST>
struct constructor_helper<ST *, bi::create_only_t, ST *>
{
	ST *m_t;
	bi::managed_shared_memory &m_memory;
	template <typename ...Args>
	constructor_helper(bi::managed_shared_memory &p_memory, const Args&... p_args) : m_memory(p_memory)
	{
		m_t = m_memory.construct<ST>(bi::unique_instance)(p_args...);
	}
	~constructor_helper()
	{
		m_memory.destroy_ptr(m_t);
	}
};


template <typename ST, typename CT>
struct constructor_initializer_helper
{
};

template <typename ST, typename CT>
struct constructor_initializer_helper<ST *, CT>
{
	ST *m_t;
	template <typename ...Args>
	constructor_initializer_helper(bi::managed_shared_memory &p_memory, const Args&... p_args)
	{
		m_t = p_memory.find<ST>(bi::unique_instance).first;
	}
};

template <typename ST>
struct constructor_initializer_helper<ST *, bi::create_only_t>
{
	ST *m_t;
	bi::managed_shared_memory &m_memory;
	template <typename ...Args>
	constructor_initializer_helper(bi::managed_shared_memory &p_memory, const Args&... p_args) : m_memory(p_memory)
	{
		m_t = m_memory.construct<ST>(bi::unique_instance)(p_args...);
		m_t->init(m_memory);
	}
	~constructor_initializer_helper()
	{
		m_t->deinit(m_memory);
		m_memory.destroy_ptr(m_t);
	}
};


}

#endif /* #ifndef _OROPO_BASE_CONSTRUCTOR_HELPER_HPP_ */

