/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_BASE_DATAFILE_CONVERTER_HPP_
#define _OROPO_BASE_DATAFILE_CONVERTER_HPP_

#include <string>
#include <boost/system/system_error.hpp>
#include <oropo/base/temporary_file.hpp>

namespace oropo
{

namespace bs=boost::system;

inline std::string get_temporary_converter_path_template()
{
	return std::string(get_temporary_path()) + std::string("/datafile_converter_XXXXXX");
}

class conversion_error : public bs::system_error
{
public:
	template <typename ...Args>
	conversion_error(Args... args) : bs::system_error(args...) {}
};

struct data_descriptor
{
	void *m_data;
	const size_t m_length;
	data_descriptor(void *p_data, const size_t p_length);
};

class datafile_converter
{
	const std::string m_tmp_template;
public:
	datafile_converter(const std::string &p_tmp_template = get_temporary_converter_path_template());

	std::string data2file(const data_descriptor &dd) throw(conversion_error);
	data_descriptor file2data(const std::string &path) throw(conversion_error);
	void freedata(const data_descriptor &dd) throw(conversion_error);
};

}

#endif /* #ifndef _OROPO_BASE_DATAFILE_CONVERTER_HPP_ */
