/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_BASE_ID_COMPOSER_HPP_
#define _OROPO_BASE_ID_COMPOSER_HPP_

#include <boost/system/system_error.hpp>
#include <oropo/base/id_composer.hpp>
#include <oropo/convert/converter.hpp>

namespace oropo
{

namespace oro=oropo;
namespace oroc=oro::convert;
namespace bs=boost::system;

template <typename ET>
class id_composer
{
private:
	std::string m_prefix;

	std::string prefix_id(const ET p_id);
public:
	id_composer() = delete;
	id_composer(std::string p_prefix);
	id_composer(const id_composer &) = delete;
	~id_composer() = default;
	id_composer &operator=(const id_composer &) = delete;

	std::string memory_name(const ET p_id);
	std::string mutex_name(const ET p_id);
	std::string producer_semaphore_name(const ET p_id);
	std::string consumer_semaphore_name(const ET p_id);
};

template <typename ET>
id_composer<ET>::id_composer(const std::string p_prefix) : m_prefix(p_prefix)
{
}

template <typename ET>
std::string id_composer<ET>::prefix_id(const ET p_id)
{
	return m_prefix+"_"+oroc::converter::to_string(p_id);
}

template <typename ET>
std::string id_composer<ET>::memory_name(const ET p_id)
{
	return prefix_id(p_id)+"_"+"memory";
}

template <typename ET>
std::string id_composer<ET>::mutex_name(const ET p_id)
{
	return prefix_id(p_id)+"_"+"mutex";
}

template <typename ET>
std::string id_composer<ET>::producer_semaphore_name(const ET p_id)
{
	return prefix_id(p_id)+"_"+"producer_semaphore";
}

template <typename ET>
std::string id_composer<ET>::consumer_semaphore_name(const ET p_id)
{
	return prefix_id(p_id)+"_"+"consumer_semaphore";
}

}

#endif /* #ifndef _OROPO_BASE_ID_COMPOSER_HPP_ */
