/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_BASE_SHARE_HPP_
#define _OROPO_BASE_SHARE_HPP_

#include <oropo/base/constructor_helper.hpp>
#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/interprocess/sync/named_mutex.hpp>

namespace oropo
{

namespace bi=boost::interprocess;

template <typename CT>
class share
{
protected:
	constructor_helper<bi::managed_shared_memory, CT, bi::shared_memory_object> m_memory;
	constructor_helper<bi::named_mutex, CT> m_mutex;
public:
	share() = delete;
	share(const char * p_memory_name, const int p_memory_size, const char *p_mutex_name);
	share(const char * p_memory_name, const char *p_mutex_name);
	share(const share<CT> &) = delete;
	~share() = default;
	share<CT> &operator=(const share<CT> &) = delete;

	bi::managed_shared_memory &get_memory();
};

template <typename CT>
share<CT>::share(const char * p_memory_name, const int p_memory_size, const char *p_mutex_name) : m_memory(p_memory_name, p_memory_size), m_mutex(p_mutex_name)
{
}

template <typename CT>
share<CT>::share(const char * p_memory_name, const char *p_mutex_name) : m_memory(p_memory_name), m_mutex(p_mutex_name)
{
}

template <typename CT>
bi::managed_shared_memory &share<CT>::get_memory()
{
	return m_memory.m_t;
}

}

#endif /* #ifndef _OROPO_BASE_SHARE_HPP_ */
