/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_CONVERT_CONVERTER_HPP_
#define _OROPO_CONVERT_CONVERTER_HPP_

#include <sstream>
#include <string>
#include <boost/system/system_error.hpp>

namespace oropo
{
namespace convert
{

namespace bs=boost::system;

class converter_error : public bs::system_error
{
public:
	template <typename ...Args>
	converter_error(Args... args) : bs::system_error(args...) {}
};

struct converter
{
	template <typename T>
	static std::string to_string(const T &p_param) throw(converter_error);
};

template <typename T>
std::string converter::to_string(const T &p_param) throw(converter_error)
{
	std::ostringstream os;
	os << p_param;
	if (os.fail()) throw converter_error(bs::error_code(EINVAL, bs::get_system_category()), std::string("converter::oparator() failed"));
	return os.str();
}

}
}

#endif /* #ifndef _OROPO_CONVERT_CONVERTER_HPP_ */
