/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _LOG_HPP_
#define _LOG_HPP_

#include <syslog.h>
#include <stdlib.h>
#include <oropo/daemon/common.hpp>

namespace oropo
{
namespace daemon
{

template <typename SV>
void syslog_open(const char *log_id)
{
}

template <>
void syslog_open<daemon_policy_debug>(const char *log_id);

template <>
void syslog_open<daemon_policy_release>(const char *log_id);

template <typename ET, typename ...Args>
void syslog_exit(ET ev, const char *format, Args ...args)
{
	syslog(LOG_EMERG, format, args...);
	exit(ev);
}

template <typename ...Args>
void syslog_err(const char *format, Args ...args)
{
	syslog(LOG_ERR, format, args...);
}

template <typename ...Args>
void syslog_info(const char *format, Args ...args)
{
	syslog(LOG_INFO, format, args...);
}

void syslog_close();

}
}

#endif /* #ifndef _LOG_HPP_ */

