/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_DAEMON_SERVICE_HPP_
#define _OROPO_DAEMON_SERVICE_HPP_

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <fstream>
#include <oropo/daemon/common.hpp>
#include <oropo/daemon/log.hpp>
#include <oropo/daemon/error.hpp>
#include <boost/filesystem.hpp>

namespace oropo
{
namespace daemon
{

namespace bf = boost::filesystem;

/*
class service
{
public:
	typedef daemon_policy_debug dp_t;
	template <typename ...Args>
	void operator()(Args ...args) {}
};
*/

template <typename DP>
int daemon(int nochdir, int noclose);

template <>
int daemon<daemon_policy_debug>(int nochdir, int noclose)
{
	return 0;
}

template <>
int daemon<daemon_policy_release>(int nochdir, int noclose)
{
	return ::daemon(0, 0);
}


template <typename S, typename ...Args>
void run_daemon(char const * p_service_id, char const * p_pid_path, S &p_service, Args ...args) throw(error)
{
	umask(0);
	syslog_open<typename S::dp_t>(p_service_id);

	if (daemon<typename S::dp_t>(0, 0)==-1)
		throw error(bs::error_code(errno, bs::get_system_category()), "daemon: ");

	bf::path pid_file_path(p_pid_path);

	std::ofstream ofs(pid_file_path.string());
	if (!ofs.good())
		throw error(bs::error_code(errno, bs::get_system_category()), "open: " + pid_file_path.string());
	ofs << getpid();
	if (!ofs.good())
		throw error(bs::error_code(errno, bs::get_system_category()), "write pid to: " + pid_file_path.string());
	ofs.close();

	p_service(args...);

	syslog_close();
}

}
}

#endif /* #ifndef _OROPO_DAEMON_SERVICE_HPP_ */
