/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _PARAMETER_ERROR_HPP_
#define _PARAMETER_ERROR_HPP_

#include <string>
#include <boost/system/system_error.hpp>
#include <ostream>

namespace oropo
{
namespace manager
{

namespace bsy=boost::system;

class parameter_error : public bsy::system_error
{
public:
	template <typename ...Args>
	parameter_error(Args... args) : bsy::system_error(args...) {}
};

enum class parameter_error_enum_t : int
{
	unexpected_type = 1,
					  wrong_string_format,
					  p2m_conversion,
					  m2p_conversion,
					  free_protocol,
					  free_resources
				  };

std::ostream &operator<<(std::ostream &p_os, const parameter_error_enum_t &p_error);

class parameter_category : public bsy::error_category
{
	const char *m_name;
public:
	parameter_category(const char *p_name = "parameter");
private:
	const char *name() const;
	std::string message(int ev) const;
};

extern parameter_category g_parameter_category;

bsy::error_code make_error_code(parameter_error_enum_t e);

}
}

namespace boost
{
namespace system
{
template<> struct is_error_code_enum<oropo::manager::parameter_error_enum_t>
{
	static const bool value = true;
};
}
}


#endif /* #ifndef _PARAMETER_ERROR_HPP_ */
