/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_MANAGER_SERIALIZATION_PARAMETER_HPP_
#define _OROPO_MANAGER_SERIALIZATION_PARAMETER_HPP_

#include <boost/serialization/string.hpp>
#include <sstream>
#include <oropo/manager/parameter.hpp>
#include <oropo/manager/serialization/common.hpp>

namespace oropo
{
namespace manager
{

namespace bse = boost::serialization;

struct param_t_wrapper
{
	param_t &m_value;
	param_t_wrapper(param_t &p_value);
};

template <typename Archive>
void serialize(Archive &ar, parameter &p_param, const unsigned int version)
{
	param_t_wrapper wrapper(p_param.m_type);
	ar & boost::serialization::make_nvp("type", wrapper);
}

template <typename Archive>
void serialize(Archive &ar, mem_parameter &p_mem_param, const unsigned int version)
{
	ar & BOOST_SERIALIZATION_BASE_OBJECT_NVP_INTRUSIVE(parameter, &p_mem_param);
	const char *value_label = "value";
	switch (p_mem_param.get_type())
	{
		case param_t::PARAM_INT:
			ar & boost::serialization::make_nvp(value_label, p_mem_param.m_int);
			break;
		case param_t::PARAM_FLOAT:
			ar & boost::serialization::make_nvp(value_label, p_mem_param.m_float);
			break;
		case param_t::PARAM_STRING:
			ar & boost::serialization::make_nvp(value_label, p_mem_param.m_string);
			break;
		case param_t::PARAM_PATH:
			ar & boost::serialization::make_nvp(value_label, p_mem_param.m_string);
			break;
		default:
			throw parameter_error(make_error_code(parameter_error_enum_t::unexpected_type), "save parameter");
	}
}

}
}

namespace boost
{
namespace serialization
{

template <typename Archive>
void save(Archive &a, oropo::manager::param_t_wrapper const &p_param, const unsigned int version)
{
	std::ostringstream os;
	os << p_param.m_value;
	std::string text = os.str();
	a & BOOST_SERIALIZATION_NVP(text);
}

template <typename Archive>
void load(Archive &a, oropo::manager::param_t_wrapper &p_param, const unsigned int version)
{
	std::string text;
	a & BOOST_SERIALIZATION_NVP(text);
	std::istringstream is(text);
	is >> p_param.m_value;
}

template <typename Archive>
void serialize(Archive &a, oropo::manager::param_t_wrapper &p_param, const unsigned int version)
{
	if (Archive::is_saving::value)
		save(a, p_param, version);
	else
		load(a, p_param, version);
}

}
}

#endif /* #ifndef _OROPO_MANAGER_SERIALIZATION_PARAMETER_HPP_ */
