/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_MANAGER_SERIALIZATION_REQUEST_HPP_
#define _OROPO_MANAGER_SERIALIZATION_REQUEST_HPP_

#include <oropo/manager/serialization/common.hpp>
#include <oropo/manager/request.hpp>
#include <oropo/manager/task.hpp>
#include <oropo/manager/serialization/task.hpp>

namespace oropo
{
namespace manager
{

namespace bse = boost::serialization;

template <typename Archive, typename E>
void serialize(Archive &ar, request<E> &p_request, const unsigned int version)
{
	typedef task<E> task_e;
	ar & BOOST_SERIALIZATION_BASE_OBJECT_NVP_INTRUSIVE(task_e, &p_request);
	ar & BOOST_SERIALIZATION_NVP(p_request.m_id);
	ar & BOOST_SERIALIZATION_NVP(p_request.m_program);
	ar & BOOST_SERIALIZATION_NVP(p_request.m_version);
	ar & BOOST_SERIALIZATION_NVP(p_request.m_procedure);
	ar & BOOST_SERIALIZATION_NVP(p_request.m_timeout);
}

template <typename Archive>
void serialize(Archive &ar, mem_request &p_mem_request, const unsigned int version)
{
	typedef request<std::vector<mem_parameter> > request_vmp;
	ar & BOOST_SERIALIZATION_BASE_OBJECT_NVP_INTRUSIVE(request_vmp, &p_mem_request);
}

}
}


#endif /* #ifndef _OROPO_MANAGER_SERIALIZATION_REQUEST_HPP_ */
