/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_MANAGER_TASK_HPP_
#define _OROPO_MANAGER_TASK_HPP_

#include <iostream>

namespace oropo
{
namespace manager
{

template <typename T>
class task
{
public:
	typedef T params_t;
protected:
	params_t m_parameters;
public:
	params_t get_parameters();
	void set_parameters(const params_t &p_parameters);

	task() = default;
	task(const params_t &p_parameters);
	task(const task &) = default;
	~task() = default;
	task &operator=(const task &) = default;

	template <typename Archive, typename E>
	friend void serialize(Archive &ar, task<E> &p_param, const unsigned int version);
	template <typename E>
	friend std::ostream &operator<<(std::ostream &os, const task<E> &p_task);
};

template <typename T>
task<T>::task(const params_t &p_parameters) : m_parameters(p_parameters) {}

template <typename T>
T task<T>::get_parameters()
{
	return m_parameters;
}

template <typename T>
void task<T>::set_parameters(const params_t &p_parameters)
{
	m_parameters = p_parameters;
}

template <typename E>
std::ostream &operator<<(std::ostream &os, const task<E> &p_task)
{
	os << "parameters: [";
	for (typename task<E>::params_t::const_iterator iter = p_task.m_parameters.begin(); iter != p_task.m_parameters.end(); iter++)
		os << *iter << " ";
	os << "] ";
	return os;
}

}
}


#endif /* #ifndef _OROPO_MANAGER_TASK_HPP_ */
