/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_MONITOR_MONITOR_HPP_
#define _OROPO_MONITOR_MONITOR_HPP_

#include <algorithm>
#include <functional>
#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/interprocess/sync/interprocess_mutex.hpp>
#include <boost/interprocess/sync/scoped_lock.hpp>
#include <oropo/monitor/monitor_data.hpp>
#include <oropo/base/constructor_helper.hpp>
#include <oropo/base/share.hpp>

namespace oropo
{

namespace monitor
{

template<typename CT>
class monitor_service : public share<CT>
{
	constructor_initializer_helper<monitor *, CT> m_monitor;
public:
	monitor_service() = delete;
	monitor_service(const char * p_memory_name, const int p_memory_size, const char *p_mutex_name);
	monitor_service(const char * p_memory_name, const char *p_mutex_name);
	monitor_service(const monitor_service &) = delete;
	~monitor_service() = default;
	monitor_service &operator=(const monitor_service &) = delete;

	void get_services(stdset &so);
	void set_services(stdset &si);
	template <typename TP>
	void select_services(TP const &p_pred, stdset &so);
};

template <typename CT>
monitor_service<CT>::monitor_service(const char * p_memory_name, const int p_memory_size, const char *p_mutex_name) : share<CT>(p_memory_name, p_memory_size, p_mutex_name), m_monitor(share<CT>::m_memory.m_t)
{
}

template <typename CT>
monitor_service<CT>::monitor_service(const char * p_memory_name, const char *p_mutex_name) : share<CT>(p_memory_name, p_mutex_name), m_monitor(share<CT>::m_memory.m_t)
{
}

template<typename CT>
void monitor_service<CT>::get_services(stdset &so)
{
	bi::managed_shared_memory &segment = share<CT>::m_memory.m_t;
	monitor &rpc = *m_monitor.m_t;
	bi::scoped_lock<bi::named_mutex> mutex(share<CT>::m_mutex.m_t);
	bi_set_to_std_set(segment, *rpc.m_services, so);
}

template<typename CT>
void monitor_service<CT>::set_services(stdset &si)
{
	bi::managed_shared_memory &segment = share<CT>::m_memory.m_t;
	monitor &rpc = *m_monitor.m_t;
	bi::scoped_lock<bi::named_mutex> mutex(share<CT>::m_mutex.m_t);
	std_set_to_bi_set(segment, si, *rpc.m_services);
}

template<typename CT>
template <typename TP>
void monitor_service<CT>::select_services(TP const &p_pred, stdset &so)
{
	stdset tmp;
	get_services(tmp);
	for (stdset::iterator iter = tmp.begin(); iter != tmp.end(); iter++)
		if (p_pred(*iter)) so.insert(*iter);
}


}

}

#endif
