/*
 * Copyright (c) 2009, 2010, Maciej Smolenski (jezdnia@gmail.com)
 * All rights reserved.
 *
 * This file is part of Oropo project.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Oropo nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OROPO_MANAGER_WORKER_DATA_HPP_
#define _OROPO_MANAGER_WORKER_DATA_HPP_

#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/interprocess/containers/set.hpp>
#include <boost/interprocess/allocators/allocator.hpp>
#include <set>
#include <oropo/worker/worker_data.hpp>

namespace oropo
{
namespace worker
{

namespace bi=boost::interprocess;

typedef std::set<mem_worker_data> stdset;
typedef bi::allocator<shm_worker_data, bi::managed_shared_memory::segment_manager> shm_worker_data_allocator;
typedef bi::set<shm_worker_data, std::less<shm_worker_data>, shm_worker_data_allocator> biset;

void std_set_to_bi_set(bi::managed_shared_memory &segment, const stdset &si, biset &so);
void bi_set_to_std_set(bi::managed_shared_memory &segment, const biset &si, stdset &so);

struct data
{
	bi::offset_ptr<biset> m_informations;

	data() = default;
	data(const data &) = delete;
	~data() = default;
	data &operator=(const data &) = delete;

	void init(bi::managed_shared_memory &p_segment);
	void deinit(bi::managed_shared_memory &p_segment);
};

}
}

#endif /* #ifndef _OROPO_MANAGER_WORKER_DATA_HPP_ */
